#
# Copyright (C) 2005-2008 by Pieter Palmers
#
# This file is part of FFADO
# FFADO = Free FireWire (pro-)audio drivers for Linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# from PyQt4.QtGui import QWidget, QHBoxLayout, QMessageBox
from ffado.import_pyqt import *

from ffado.config import *
from ffado.mixer.saffire_base import SaffireMixerBase

import logging
log = logging.getLogger('saffirepro')

class SaffirePro(QWidget):
    def __init__(self,parent = None):
        QWidget.__init__(self, parent)

        self.have_adat = False
        self.samplerate = None
        self.is_pro10 = None

        # make a layout
        self.layout = QHBoxLayout()
        self.setLayout(self.layout)

    def show(self):
        self.selectCorrectMode()
        QWidget.show(self)

    def getDisplayTitle(self):
        try:
            return self.nickname.text()
        except:
            if self.is_pro10:
                return "SaffirePRO10"
            else:
                return "SaffirePRO26"

    def selectCorrectMode(self):
        if self.samplerate <= 96000:
            log.debug("large")
            self.small.hide()
            self.large.initValues()
            self.large.show()
        else:
            log.debug("small")
            self.large.hide()
            self.small.initValues()
            self.small.show()

    def initValues(self):
        self.is_not_streaming = self.samplerateselect.canChangeValue()
        selected = self.samplerateselect.selected()
        self.samplerate = int(self.samplerateselect.getEnumLabel( selected ))
        log.debug("detected samplerate %d" % self.samplerate)

        # adat on PRO26 makes a difference
        modelId = self.configrom.getModelId()
        if modelId == 0x00000003: # PRO26
            self.is_pro10 = False
            state = self.hw.getDiscrete('/Control/ADATDisable')
            if state:
                self.have_adat = False
                log.debug("detected PRO26, ADAT disabled")
            else:
                self.have_adat = True
                log.debug("detected PRO26, ADAT enabled")
        elif modelId == 0x00000006: # PRO10
            self.is_pro10 = True
            self.have_adat = False

        # create the child widgets
        self.small = SaffireProMixerSmall(self)
        self.layout.addWidget(self.small)
        self.large = SaffireProMixerLarge(self)
        self.layout.addWidget(self.large)

        self.small.hw = self.hw
        self.small.configrom = self.configrom

        self.large.hw = self.hw
        self.large.configrom = self.configrom

        self.selectCorrectMode()

    def polledUpdate(self):
        self.is_not_streaming = self.samplerateselect.canChangeValue()
        if self.samplerate <= 96000:
            self.large.polledUpdate()
        else:
            self.small.polledUpdate()

class SaffireProMixerLarge(QWidget, SaffireMixerBase):
    def __init__(self,parent = None):
        self.my_parent = parent
        QWidget.__init__(self,parent)
        SaffireMixerBase.__init__(self)
        uicLoad("ffado/mixer/saffirepro_large", self)
        self.have_adat = False
        log.debug("Init large Saffire Pro mixer window")

        self.VolumeControls={
            self.sldIMixAnalog1L: ['/Mixer/InputMix', 0, 0], 
            self.sldIMixAnalog1R: ['/Mixer/InputMix', 0, 1], 
            self.sldIMixAnalog2L: ['/Mixer/InputMix', 1, 0], 
            self.sldIMixAnalog2R: ['/Mixer/InputMix', 1, 1], 
            self.sldIMixAnalog3L: ['/Mixer/InputMix', 2, 0], 
            self.sldIMixAnalog3R: ['/Mixer/InputMix', 2, 1], 
            self.sldIMixAnalog4L: ['/Mixer/InputMix', 3, 0], 
            self.sldIMixAnalog4R: ['/Mixer/InputMix', 3, 1], 
            self.sldIMixAnalog5L: ['/Mixer/InputMix', 4, 0], 
            self.sldIMixAnalog5R: ['/Mixer/InputMix', 4, 1], 
            self.sldIMixAnalog6L: ['/Mixer/InputMix', 5, 0], 
            self.sldIMixAnalog6R: ['/Mixer/InputMix', 5, 1], 
            self.sldIMixAnalog7L: ['/Mixer/InputMix', 6, 0], 
            self.sldIMixAnalog7R: ['/Mixer/InputMix', 6, 1], 
            self.sldIMixAnalog8L: ['/Mixer/InputMix', 7, 0], 
            self.sldIMixAnalog8R: ['/Mixer/InputMix', 7, 1], 
            self.sldIMixAnalog9L: ['/Mixer/InputMix', 8, 0], 
            self.sldIMixAnalog9R: ['/Mixer/InputMix', 8, 1], 
            self.sldIMixAnalog10L: ['/Mixer/InputMix', 9, 0], 
            self.sldIMixAnalog10R: ['/Mixer/InputMix', 9, 1], 
            self.sldIMixADAT11L: ['/Mixer/InputMix', 10, 0], 
            self.sldIMixADAT11R: ['/Mixer/InputMix', 10, 1], 
            self.sldIMixADAT12L: ['/Mixer/InputMix', 11, 0], 
            self.sldIMixADAT12R: ['/Mixer/InputMix', 11, 1], 
            self.sldIMixADAT13L: ['/Mixer/InputMix', 12, 0], 
            self.sldIMixADAT13R: ['/Mixer/InputMix', 12, 1], 
            self.sldIMixADAT14L: ['/Mixer/InputMix', 13, 0], 
            self.sldIMixADAT14R: ['/Mixer/InputMix', 13, 1], 
            self.sldIMixADAT15L: ['/Mixer/InputMix', 14, 0], 
            self.sldIMixADAT15R: ['/Mixer/InputMix', 14, 1], 
            self.sldIMixADAT16L: ['/Mixer/InputMix', 15, 0], 
            self.sldIMixADAT16R: ['/Mixer/InputMix', 15, 1], 
            self.sldIMixADAT17L: ['/Mixer/InputMix', 16, 0], 
            self.sldIMixADAT17R: ['/Mixer/InputMix', 16, 1], 
            self.sldIMixADAT18L: ['/Mixer/InputMix', 17, 0], 
            self.sldIMixADAT18R: ['/Mixer/InputMix', 17, 1], 
            self.sldIMixADAT21L: ['/Mixer/InputMix', 18, 0], 
            self.sldIMixADAT21R: ['/Mixer/InputMix', 18, 1], 
            self.sldIMixADAT22L: ['/Mixer/InputMix', 19, 0], 
            self.sldIMixADAT22R: ['/Mixer/InputMix', 19, 1], 
            self.sldIMixADAT23L: ['/Mixer/InputMix', 20, 0], 
            self.sldIMixADAT23R: ['/Mixer/InputMix', 20, 1], 
            self.sldIMixADAT24L: ['/Mixer/InputMix', 21, 0], 
            self.sldIMixADAT24R: ['/Mixer/InputMix', 21, 1], 
            self.sldIMixADAT25L: ['/Mixer/InputMix', 22, 0], 
            self.sldIMixADAT25R: ['/Mixer/InputMix', 22, 1], 
            self.sldIMixADAT26L: ['/Mixer/InputMix', 23, 0], 
            self.sldIMixADAT26R: ['/Mixer/InputMix', 23, 1], 
            self.sldIMixADAT27L: ['/Mixer/InputMix', 24, 0], 
            self.sldIMixADAT27R: ['/Mixer/InputMix', 24, 1], 
            self.sldIMixADAT28L: ['/Mixer/InputMix', 25, 0], 
            self.sldIMixADAT28R: ['/Mixer/InputMix', 25, 1],
            
            self.sldOMixPC1O1: ['/Mixer/OutputMix', 0, 0], 
            self.sldOMixPC2O2: ['/Mixer/OutputMix', 1, 1], 
            self.sldOMixPC3O3: ['/Mixer/OutputMix', 2, 2], 
            self.sldOMixPC4O4: ['/Mixer/OutputMix', 3, 3], 
            self.sldOMixPC5O5: ['/Mixer/OutputMix', 4, 4], 
            self.sldOMixPC6O6: ['/Mixer/OutputMix', 5, 5], 
            self.sldOMixPC7O7: ['/Mixer/OutputMix', 6, 6], 
            self.sldOMixPC8O8: ['/Mixer/OutputMix', 7, 7], 
            self.sldOMixPC9O9: ['/Mixer/OutputMix', 8, 8], 
            self.sldOMixPC10O10: ['/Mixer/OutputMix', 9, 9],
            
            self.sldOMixPC1O3: ['/Mixer/OutputMix', 0, 2], 
            self.sldOMixPC2O4: ['/Mixer/OutputMix', 1, 3], 
            self.sldOMixPC1O5: ['/Mixer/OutputMix', 0, 4], 
            self.sldOMixPC2O6: ['/Mixer/OutputMix', 1, 5], 
            self.sldOMixPC1O7: ['/Mixer/OutputMix', 0, 6], 
            self.sldOMixPC2O8: ['/Mixer/OutputMix', 1, 7], 
            self.sldOMixPC1O9: ['/Mixer/OutputMix', 0, 8], 
            self.sldOMixPC2O10: ['/Mixer/OutputMix', 1, 9], 
            
            self.sldOMixIMixO1: ['/Mixer/OutputMix', 10, 0], 
            self.sldOMixIMixO2: ['/Mixer/OutputMix', 11, 1], 
            self.sldOMixIMixO3: ['/Mixer/OutputMix', 10, 2], 
            self.sldOMixIMixO4: ['/Mixer/OutputMix', 11, 3], 
            self.sldOMixIMixO5: ['/Mixer/OutputMix', 10, 4], 
            self.sldOMixIMixO6: ['/Mixer/OutputMix', 11, 5], 
            self.sldOMixIMixO7: ['/Mixer/OutputMix', 10, 6], 
            self.sldOMixIMixO8: ['/Mixer/OutputMix', 11, 7], 
            self.sldOMixIMixO9: ['/Mixer/OutputMix', 10, 8], 
            self.sldOMixIMixO10: ['/Mixer/OutputMix', 11, 9], 
            }


        self.SelectorControls={
            # control elements
            self.chkInsert1: ['/Control/Insert1'], 
            self.chkInsert2: ['/Control/Insert2'], 
            self.chkPhantom14: ['/Control/Phantom_1to4'], 
            self.chkPhantom58: ['/Control/Phantom_5to8'], 
            self.chkAC3: ['/Control/AC3pass'], 
            self.chkMidiThru: ['/Control/MidiTru'], 
            self.chkHighVoltage: ['/Control/UseHighVoltageRail'], 
            #self.chkEnableADAT1: ['/Control/EnableAdat1'], 
            #self.chkEnableADAT2: ['/Control/EnableAdat2'],
            #self.chkEnableSPDIF1: ['/Control/EnableSPDIF1'],
            self.chkMidiEnable: ['/Control/MIDIEnable'],
            self.chkAdatDisable: ['/Control/ADATDisable'],
            # Mixer switches
            self.chkMute12: ['/Mixer/Out12Mute', [self.chkHwCtrl12]],
            self.chkHwCtrl12: ['/Mixer/Out12HwCtrl'],
            self.chkPad12: ['/Mixer/Out12Pad'],
            self.chkDim12: ['/Mixer/Out12Dim'],
            self.chkMute34: ['/Mixer/Out34Mute', [self.chkHwCtrl34]],
            self.chkHwCtrl34: ['/Mixer/Out34HwCtrl'],
            self.chkPad34: ['/Mixer/Out34Pad'],
            self.chkDim34: ['/Mixer/Out34Dim'],
            self.chkMute56: ['/Mixer/Out56Mute', [self.chkHwCtrl56]],
            self.chkHwCtrl56: ['/Mixer/Out56HwCtrl'],
            self.chkPad56: ['/Mixer/Out56Pad'],
            self.chkDim56: ['/Mixer/Out56Dim'],
            self.chkMute78: ['/Mixer/Out78Mute', [self.chkHwCtrl78]],
            self.chkHwCtrl78: ['/Mixer/Out78HwCtrl'],
            self.chkPad78: ['/Mixer/Out78Pad'],
            self.chkDim78: ['/Mixer/Out78Dim'],
            # direct monitoring
            self.chkMonitor1: ['/Mixer/DirectMonitorCH1'],
            self.chkMonitor2: ['/Mixer/DirectMonitorCH2'],
            self.chkMonitor3: ['/Mixer/DirectMonitorCH3'],
            self.chkMonitor4: ['/Mixer/DirectMonitorCH4'],
            self.chkMonitor5: ['/Mixer/DirectMonitorCH5'],
            self.chkMonitor6: ['/Mixer/DirectMonitorCH6'],
            self.chkMonitor7: ['/Mixer/DirectMonitorCH7'],
            self.chkMonitor8: ['/Mixer/DirectMonitorCH8'],
        }

        self.VolumeControlsLowRes={
            self.sldOut12Level:      ['/Mixer/Out12Level'],
            self.sldOut34Level:      ['/Mixer/Out34Level'],
            self.sldOut56Level:      ['/Mixer/Out56Level'],
            self.sldOut78Level:      ['/Mixer/Out78Level'],
        }

        self.TriggerButtonControls={
            self.btnReboot:        ['/Control/Reboot'],
            self.btnIdentify:      ['/Control/FlashLed'],
            self.btnSaveSettings:  ['/Control/SaveSettings'],
        }

        self.TextControls={
        }

        self.saveTextControls={
        }

        self.ComboControls={
            self.comboStandalone:        ['/Control/StandaloneConfig'],
        }

    def updateMatrixVolume(self,a0):
        SaffireMixerBase.updateMatrixVolume(self,a0)
    def updateLowResVolume(self,a0):
        SaffireMixerBase.updateLowResVolume(self,a0)
    def updateSelector(self,a0):
        sender = self.sender()
        #if sender == self.chkAC3 and not self.my_parent.is_not_streaming:
            #msg = QMessageBox()
            #msg.question( msg, "Error", \
                #"<qt>Change not permitted. Is streaming active?</qt>", \
                #QMessageBox.Ok )
            #self.chkAC3.setEnabled(False)
            #if a0:
                #self.chkAC3.setChecked(False)
            #else:
                #self.chkAC3.setChecked(True)
            #return
        if sender == self.chkMidiEnable and not self.my_parent.is_not_streaming:
            msg = QMessageBox()
            msg.question( msg, "Error", \
                "<qt>Change not permitted. Is streaming active?</qt>", \
                QMessageBox.Ok )
            self.chkMidiEnable.setEnabled(False)
            state = self.hw.getDiscrete(self.SelectorControls[self.chkMidiEnable][0])
            if state:
                self.chkMidiEnable.setChecked(True)
            else:
                self.chkMidiEnable.setChecked(False)
            return
        if sender == self.chkAdatDisable and not self.my_parent.is_not_streaming:
            msg = QMessageBox()
            msg.question( msg, "Error", \
                "<qt>Change not permitted. Is streaming active?</qt>", \
                QMessageBox.Ok )
            self.chkAdatDisable.setEnabled(False)
            state = self.hw.getDiscrete(self.SelectorControls[self.chkAdatDisable][0])
            if state:
                self.chkAdatDisable.setChecked(True)
            else:
                self.chkAdatDisable.setChecked(False)
            return
        SaffireMixerBase.updateSelector(self,a0)

    def triggerButton(self):
        sender = self.sender()
        if sender == self.btnReboot and not self.my_parent.is_not_streaming:
            msg = QMessageBox()
            msg.question( msg, "Error", \
                "<qt>Operation not permitted. Is streaming active?</qt>", \
                QMessageBox.Ok )
            self.btnReboot.setEnabled(False)
            return
        SaffireMixerBase.triggerButton(self)

    def saveText(self):
        SaffireMixerBase.saveText(self)
    def initCombo(self, combo):
        SaffireMixerBase.initCombo(self,combo)
    def selectCombo(self, mode):
        SaffireMixerBase.selectCombo(self,mode)

    def updateValues(self):
        for i in range(self.tabInputMix.count()):
            self.tabInputMix.setTabEnabled(i, True)

        if not self.my_parent.have_adat:
            for i in range(self.tabInputMix.count()):
                page = self.tabInputMix.widget(i)
                name = page.objectName()
                if name[0:4] == "adat":
                    self.tabInputMix.setTabEnabled(i, False)
                else:
                    self.tabInputMix.setTabEnabled(i, True)

        self.tabInputMix.setCurrentWidget(self.tabInputMix.widget(0))
        SaffireMixerBase.updateValues(self)

    def polledUpdate(self):
        #log.debug("polled update (large)")
        self.polledUpdateHwCtrl(self.chkHwCtrl12, self.sldOut12Level)
        self.polledUpdateHwCtrl(self.chkHwCtrl34, self.sldOut34Level)
        self.polledUpdateHwCtrl(self.chkHwCtrl56, self.sldOut56Level)
        self.polledUpdateHwCtrl(self.chkHwCtrl78, self.sldOut78Level)

        #make these inaccessible whenever streaming is running
        #self.chkAC3.setEnabled(self.my_parent.is_not_streaming)
        self.chkMidiEnable.setEnabled(self.my_parent.is_not_streaming)
        self.chkAdatDisable.setEnabled(self.my_parent.is_not_streaming)
        self.btnReboot.setEnabled(self.my_parent.is_not_streaming)

    def polledUpdateHwCtrl(self, selector, volctrl):
        state = selector.isChecked()
        if state:
            self.polledUpdateVolumeLowRes('/Mixer/MonitorDial', volctrl, 2)
            volctrl.setEnabled(False)
        else:
            volctrl.setEnabled(True)

class SaffireProMixerSmall(QWidget, SaffireMixerBase):
    def __init__(self,parent = None):
        self.my_parent = parent
        QWidget.__init__(self,parent)
        SaffireMixerBase.__init__(self)
        uicLoad("ffado/mixer/saffirepro_small", self)
        log.debug("Init small Saffire Pro mixer window")

        self.VolumeControls={

            self.sldOMixPC1O1: ['/Mixer/OutputMix', 0, 0], 
            self.sldOMixPC2O2: ['/Mixer/OutputMix', 1, 1], 
            self.sldOMixPC3O3: ['/Mixer/OutputMix', 2, 2], 
            self.sldOMixPC4O4: ['/Mixer/OutputMix', 3, 3], 
            self.sldOMixPC5O5: ['/Mixer/OutputMix', 4, 4], 
            self.sldOMixPC6O6: ['/Mixer/OutputMix', 5, 5], 
            self.sldOMixPC7O7: ['/Mixer/OutputMix', 6, 6], 
            self.sldOMixPC8O8: ['/Mixer/OutputMix', 7, 7], 
            self.sldOMixPC9O9: ['/Mixer/OutputMix', 8, 8], 
            self.sldOMixPC10O10: ['/Mixer/OutputMix', 9, 9],
            
            self.sldOMixPC1O3: ['/Mixer/OutputMix', 0, 2], 
            self.sldOMixPC2O4: ['/Mixer/OutputMix', 1, 3], 
            self.sldOMixPC1O5: ['/Mixer/OutputMix', 0, 4], 
            self.sldOMixPC2O6: ['/Mixer/OutputMix', 1, 5], 
            self.sldOMixPC1O7: ['/Mixer/OutputMix', 0, 6], 
            self.sldOMixPC2O8: ['/Mixer/OutputMix', 1, 7], 
            self.sldOMixPC1O9: ['/Mixer/OutputMix', 0, 8], 
            self.sldOMixPC2O10: ['/Mixer/OutputMix', 1, 9], 
            
            self.sldOMixIMixO1: ['/Mixer/OutputMix', 10, 0], 
            self.sldOMixIMixO2: ['/Mixer/OutputMix', 11, 1], 
            self.sldOMixIMixO3: ['/Mixer/OutputMix', 10, 2], 
            self.sldOMixIMixO4: ['/Mixer/OutputMix', 11, 3], 
            self.sldOMixIMixO5: ['/Mixer/OutputMix', 10, 4], 
            self.sldOMixIMixO6: ['/Mixer/OutputMix', 11, 5], 
            self.sldOMixIMixO7: ['/Mixer/OutputMix', 10, 6], 
            self.sldOMixIMixO8: ['/Mixer/OutputMix', 11, 7], 
            self.sldOMixIMixO9: ['/Mixer/OutputMix', 10, 8], 
            self.sldOMixIMixO10: ['/Mixer/OutputMix', 11, 9], 
            }


        self.SelectorControls={
            # control elements
            self.chkInsert1: ['/Control/Insert1'], 
            self.chkInsert2: ['/Control/Insert2'], 
            self.chkPhantom14: ['/Control/Phantom_1to4'], 
            self.chkPhantom58: ['/Control/Phantom_5to8'], 
            self.chkAC3: ['/Control/AC3pass'], 
            self.chkMidiThru: ['/Control/MidiTru'], 
            self.chkHighVoltage: ['/Control/UseHighVoltageRail'], 
            #self.chkEnableADAT1: ['/Control/EnableAdat1'], 
            #self.chkEnableADAT2: ['/Control/EnableAdat2'],
            #self.chkEnableSPDIF1: ['/Control/EnableSPDIF1'],
            self.chkMidiEnable: ['/Control/MIDIEnable'],
            self.chkAdatDisable: ['/Control/ADATDisable'],
            # Mixer switches
            self.chkMute12: ['/Mixer/Out12Mute'],
            self.chkHwCtrl12: ['/Mixer/Out12HwCtrl'],
            self.chkPad12: ['/Mixer/Out12Pad'],
            self.chkDim12: ['/Mixer/Out12Dim'],
            self.chkMute34: ['/Mixer/Out34Mute'],
            self.chkHwCtrl34: ['/Mixer/Out34HwCtrl'],
            self.chkPad34: ['/Mixer/Out34Pad'],
            self.chkDim34: ['/Mixer/Out34Dim'],
            self.chkMute56: ['/Mixer/Out56Mute'],
            self.chkHwCtrl56: ['/Mixer/Out56HwCtrl'],
            self.chkPad56: ['/Mixer/Out56Pad'],
            self.chkDim56: ['/Mixer/Out56Dim'],
            self.chkMute78: ['/Mixer/Out78Mute'],
            self.chkHwCtrl78: ['/Mixer/Out78HwCtrl'],
            self.chkPad78: ['/Mixer/Out78Pad'],
            self.chkDim78: ['/Mixer/Out78Dim'],
            # direct monitoring
            self.chkMonitor1: ['/Mixer/DirectMonitorCH1'],
            self.chkMonitor2: ['/Mixer/DirectMonitorCH2'],
            self.chkMonitor3: ['/Mixer/DirectMonitorCH3'],
            self.chkMonitor4: ['/Mixer/DirectMonitorCH4'],
            self.chkMonitor5: ['/Mixer/DirectMonitorCH5'],
            self.chkMonitor6: ['/Mixer/DirectMonitorCH6'],
            self.chkMonitor7: ['/Mixer/DirectMonitorCH7'],
            self.chkMonitor8: ['/Mixer/DirectMonitorCH8'],
        }

        self.VolumeControlsLowRes={
            self.sldOut12Level:      ['/Mixer/Out12Level'],
            self.sldOut34Level:      ['/Mixer/Out34Level'],
            self.sldOut56Level:      ['/Mixer/Out56Level'],
            self.sldOut78Level:      ['/Mixer/Out78Level'],
        }

        self.TriggerButtonControls={
            self.btnReboot:        ['/Control/Reboot'],
            self.btnIdentify:      ['/Control/FlashLed'],
            self.btnSaveSettings:  ['/Control/SaveSettings'],
        }

        self.TextControls={
        }

        self.saveTextControls={
        }

        self.ComboControls={
            self.comboStandalone:        ['/Control/StandaloneConfig'],
        }

    def updateMatrixVolume(self,a0):
        SaffireMixerBase.updateMatrixVolume(self,a0)
    def updateLowResVolume(self,a0):
        SaffireMixerBase.updateLowResVolume(self,a0)
    def updateSelector(self,a0):
        sender = self.sender()
        #if sender == self.chkAC3 and not self.my_parent.is_not_streaming:
            #msg = QMessageBox()
            #msg.question( msg, "Error", \
                #"<qt>Change not permitted. Is streaming active?</qt>", \
                #QMessageBox.Ok )
            #self.chkAC3.setEnabled(False)
            #if a0:
                #self.chkAC3.setChecked(False)
            #else:
                #self.chkAC3.setChecked(True)
            #return
        if sender == self.chkMidiEnable and not self.my_parent.is_not_streaming:
            msg = QMessageBox()
            msg.question( msg, "Error", \
                "<qt>Change not permitted. Is streaming active?</qt>", \
                QMessageBox.Ok )
            self.chkMidiEnable.setEnabled(False)
            state = self.hw.getDiscrete(self.SelectorControls[self.chkMidiEnable][0])
            if state:
                self.chkMidiEnable.setChecked(True)
            else:
                self.chkMidiEnable.setChecked(False)
            return
        if sender == self.chkAdatDisable and not self.my_parent.is_not_streaming:
            msg = QMessageBox()
            msg.question( msg, "Error", \
                "<qt>Change not permitted. Is streaming active?</qt>", \
                QMessageBox.Ok )
            self.chkAdatDisable.setEnabled(False)
            state = self.hw.getDiscrete(self.SelectorControls[self.chkAdatDisable][0])
            if state:
                self.chkAdatDisable.setChecked(True)
            else:
                self.chkAdatDisable.setChecked(False)
            return
        SaffireMixerBase.updateSelector(self,a0)

    def triggerButton(self):
        sender = self.sender()
        if sender == self.btnReboot and not self.my_parent.is_not_streaming:
            msg = QMessageBox()
            msg.question( msg, "Error", \
                "<qt>Operation not permitted. Is streaming active?</qt>", \
                QMessageBox.Ok )
            self.btnReboot.setEnabled(False)
            return
        SaffireMixerBase.triggerButton(self)

    def saveText(self):
        SaffireMixerBase.saveText(self)
    def initCombo(self, combo):
        SaffireMixerBase.initCombo(self,combo)
    def selectCombo(self, mode):
        SaffireMixerBase.selectCombo(self,mode)

    def updateValues(self):
        SaffireMixerBase.updateValues(self)

    def polledUpdate(self):
        #log.debug("polled update (small)")
        self.polledUpdateHwCtrl(self.chkHwCtrl12, self.sldOut12Level)
        self.polledUpdateHwCtrl(self.chkHwCtrl34, self.sldOut34Level)
        self.polledUpdateHwCtrl(self.chkHwCtrl56, self.sldOut56Level)
        self.polledUpdateHwCtrl(self.chkHwCtrl78, self.sldOut78Level)

        #make these inaccessible whenever streaming is running
        #self.chkAC3.setEnabled(self.my_parent.is_not_streaming)
        self.chkMidiEnable.setEnabled(self.my_parent.is_not_streaming)
        self.chkAdatDisable.setEnabled(self.my_parent.is_not_streaming)
        self.btnReboot.setEnabled(self.my_parent.is_not_streaming)

    def polledUpdateHwCtrl(self, selector, volctrl):
        state = selector.isChecked()
        if state:
            self.polledUpdateVolumeLowRes('/Mixer/MonitorDial', volctrl, 2)
            volctrl.setEnabled(False)
        else:
            volctrl.setEnabled(True)

# vim: et



#
# Copyright (C) 2008 by Arnold Krille
#
# This file is part of FFADO
# FFADO = Free FireWire (pro-)audio drivers for Linux
#
# FFADO is based upon FreeBoB.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# from PyQt4.QtGui import QTextEdit, QAbstractSlider, QColor
# from PyQt4.QtCore import QObject, pyqtSignal, QString
from ffado.import_pyqt import *

import logging
log = logging.getLogger('logginghandler')

class QStatusLogger( QObject, logging.Handler ):
    log = pyqtSignal(QString if ffado_pyqt_version == 4 else str, int, name='log')
    def __init__( self, parent, statusbar, level=logging.NOTSET ):
        QObject.__init__( self, parent )
        logging.Handler.__init__( self, level )
        self.setFormatter( logging.Formatter( "%(name)s: %(message)s" ) )
        self.log.connect(statusbar.showMessage)

    def emit( self, record ):
        self.log.emit('%s: %s'.format(record.name, record.getMessage()), 5000)

class QTextLogger( logging.Handler ):
    def __init__( self, parent, level=logging.NOTSET ):
        logging.Handler.__init__( self, level )

        self.textedit = QTextEdit( parent )

        self.textedit.setReadOnly( True )
        self.textedit.setAcceptRichText( True )

    def emit( self, record ):
        color = QColor( "#000000" )
        if record.levelno > 20:
            color = QColor( "#ffff00" )
        if record.levelno > 30:
            color = QColor( "#ff0000" )
        if record.levelno <= 10:
            color = QColor( "#808080" )
        self.textedit.setTextColor( color )
        tmp = "%s %s: %s" % (record.asctime, record.name, record.getMessage())
        self.textedit.append( tmp )
        self.textedit.verticalScrollBar().triggerAction( QAbstractSlider.SliderToMaximum )

#
# vim: et
#



#
# presonus_firebox.py - Qt4/FFADO application for Presonus FIREBOX
# Copyright (C) 2013 Takashi Sakamoto <o-takashi@sakamocchi.jp>
#
# This file is part of FFADO
# FFADO = Free FireWire (pro-)audio drivers for Linux
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# version 3 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# from PyQt4 import QtGui, QtCore
# from PyQt4.QtCore import Qt
# from PyQt4.QtGui import QHBoxLayout, QVBoxLayout, QGridLayout
# from PyQt4.QtGui import QWidget, QTabWidget, QGroupBox, QLabel, QSizePolicy, QToolButton, QSlider, QComboBox, QSpacerItem, QDial
from ffado.import_pyqt import *

from math import log10
from ffado.config import *

class Presonus_Firebox(QWidget):
    #name/feature_id/+12dB_id
    inputs = [["Analog in 1/2",  5, 10],
              ["Analog in 3/4",  6, 11],
              ["Digital in 1/2", 7,  0],
              ["Stream in",      8,  0]]

    # name/id for mixer sorce selector
    streams = [["Stream in 1/2", 0],
               ["Stream in 3/4", 1],
               ["Stream in 5/6", 2],
               ["Stream in 7/8", 3]]

    # name/feature id
    outputs = [["Analog out 1/2",    1],
               ["Analog out 3/4",    2],
               ["Analog out 5/6",    3],
               ["Digital out 1/2",   0],
               ["Headphone out 1/2", 4]]

    # selector_id/sources
    out_src = [[1, ["Stream in 1/2", "Mixer out 1/2"]],
               [2, ["Stream in 3/4", "Mixer out 1/2"]],
               [3, ["Stream in 5/6", "Mixer out 1/2"]],
               [5, ["Stream in 7/8", "Mixer out 1/2"]],
               [4, ["Stream in 1/2", "Stream in 3/4",
                    "Stream in 5/6", "Stream in 7/8", "Mixer out 1/2"]]]

    def __init__(self, parent=None):
        QWidget.__init__(self, parent)

    def getDisplayTitle(self):
        return 'Firebox'

    def buildMixer(self):
        self.Selectors = {}
        self.MicBoosts = {}
        self.Volumes = {}
        self.Mutes = {}
        self.Balances = {}

        tabs = QTabWidget(self)
        tabs_layout = QHBoxLayout(self)
        tabs_layout.addWidget(tabs)

        self.addMixer(tabs)
        self.addOutput(tabs)

    def addMixer(self, tabs):
        tab_mixer = QWidget(tabs)
        tabs.addTab(tab_mixer, "Mixer")
        tab_mixer_layout = QGridLayout()
        tab_mixer.setLayout(tab_mixer_layout)

        for i in range(len(self.inputs)):
            col = 2 * i;
            label = QLabel(tab_mixer)
            tab_mixer_layout.addWidget(label, 0, col, 1, 2, Qt.AlignHCenter)

            label.setText(self.inputs[i][0])
            label.setAlignment(Qt.AlignCenter)
            label.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)

            link = self.getLink(tab_mixer)
            tab_mixer_layout.addWidget(link, 3, col, 1, 2, Qt.AlignHCenter)

            if self.inputs[i][2] > 0:
                for j in range(1, 3):
                    button = QToolButton(tab_mixer)
                    tab_mixer_layout.addWidget(button, 1, col + j - 1, Qt.AlignHCenter)
                    button.setText('+12dB')
                    button.setCheckable(True)
                    button.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed)
                    path = "/Mixer/Feature_Volume_%d" % self.inputs[i][2]
                    self.MicBoosts[button] = [path, j]

            elif self.inputs[i][0].find('Stream') >= 0:
                cmb = QComboBox(tab_mixer)
                tab_mixer_layout.addWidget(cmb, 1, col, 1, 2, Qt.AlignHCenter)
                for i in range(len(self.streams)):
                    cmb.addItem(self.streams[i][0], self.streams[i][1])
                self.Selectors[cmb] = ["/Mixer/Selector_6"]

            l_sld = self.getSlider(tab_mixer)
            r_sld = self.getSlider(tab_mixer)
            tab_mixer_layout.addWidget(l_sld, 2, col, Qt.AlignHCenter)
            tab_mixer_layout.addWidget(r_sld, 2, col + 1, Qt.AlignHCenter)

            mute = self.getMute(tab_mixer)
            tab_mixer_layout.addWidget(mute, 4, col, 1, 2, Qt.AlignHCenter)

            path = "/Mixer/Feature_Volume_%d" % self.inputs[i][1]
            self.Volumes[l_sld] = [path, 1, r_sld, link, mute]
            self.Volumes[r_sld] = [path, 2, l_sld, link, mute]
            self.Mutes[mute] = [path, l_sld, r_sld]

            for j in range(0, 2):
                dial = self.getDial(tab_mixer)
                tab_mixer_layout.addWidget(dial, 5, col + j, Qt.AlignHCenter)
                if self.inputs[i][2] > 0:
                    path = "/Mixer/Feature_LRBalance_%d" % self.inputs[i][1]
                    self.Balances[dial] = [path, j + 1]
                # to keep width
                else:
                    dial.setDisabled(True)

    def addOutput(self, tabs):
        tab_out = QWidget(self)
        tabs.addTab(tab_out, "Output")
        tab_out_layout = QGridLayout()
        tab_out.setLayout(tab_out_layout)

        for i in range(len(self.outputs)):
            col = 2 * i

            label = QLabel(tab_out)
            tab_out_layout.addWidget(label, 0, col, 1, 2, Qt.AlignCenter)
            label.setText(self.outputs[i][0])

            cmb = QComboBox(tab_out)
            tab_out_layout.addWidget(cmb, 1, col, 1, 2, Qt.AlignHCenter)
            for j in range(len(self.out_src[i][1])):
                cmb.addItem(self.out_src[i][1][j])
            self.Selectors[cmb] = ["/Mixer/Selector_%d" % self.out_src[i][0]]

            if self.outputs[i][1] == 0:
                continue

            l_sld = self.getSlider(tab_out)
            r_sld = self.getSlider(tab_out)
            tab_out_layout.addWidget(l_sld, 2, col, Qt.AlignHCenter)
            tab_out_layout.addWidget(r_sld, 2, col + 1, Qt.AlignHCenter)

            link = self.getLink(tab_out)
            tab_out_layout.addWidget(link, 3, col, 1, 2, Qt.AlignHCenter)

            mute = self.getMute(tab_out)
            tab_out_layout.addWidget(mute, 4, col, 1, 2, Qt.AlignHCenter)

            path = "/Mixer/Feature_Volume_%d" % self.outputs[i][1]
            self.Volumes[l_sld] = [path, 1, r_sld, link, mute]
            self.Volumes[r_sld] = [path, 2, l_sld, link, mute]
            self.Mutes[mute] = [path, l_sld, r_sld]

    def getSlider(self, parent):
        sld = QSlider(parent)
        sld.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Minimum)
        sld.setMinimum(0)
        sld.setMaximum(99)
        sld.setPageStep(10)
        sld.setPageStep(10)
        sld.setMinimumHeight(50)
        sld.setTickInterval(25)
        sld.setTickPosition(QSlider.TicksBothSides)
        return sld

    def getDial(self, parent):
        dial = QDial(parent)
        dial.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)
        dial.setMinimumSize(50, 50)
        dial.setMaximumHeight(50)
        dial.setNotchTarget(25)
        dial.setNotchesVisible(True)
        dial.setMinimum(0)
        dial.setMaximum(99)
        dial.setPageStep(10)
        dial.setPageStep(10)
        return dial

    def getLink(self, parent):
        link = QToolButton(parent)
        link.setText("link")
        link.setCheckable(True)
        link.setMinimumWidth(100)
        link.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed)
        return link

    def getMute(self, parent):
        mute = QToolButton(parent)
        mute.setText("mute")
        mute.setCheckable(True)
        mute.setMinimumWidth(100)
        mute.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed)
        return mute

    def initValues(self):
        for elm, params in self.Selectors.items():
            path = params[0]
            state = self.hw.getDiscrete(path)
            elm.setCurrentIndex(state)
            elm.activated.connect(self.updateSelector)

        for elm, params in self.MicBoosts.items():
            path = params[0]
            idx  = params[1]
            value = self.hw.getContignuous(path, idx)
            elm.setChecked(not value == 0)
            elm.clicked.connect(self.updateMicBoost)

        for elm, params in self.Volumes.items():
            path = params[0]
            idx  = params[1]
            db   = self.hw.getContignuous(path, idx)
            vol  = self.db2vol(db)
            elm.setValue(int(vol))
            elm.valueChanged.connect(self.updateVolume)

            if idx == 0:
                continue

            p_idx = 0
            pair = params[2]
            link = params[3]
            mute = params[4]

            p_db = self.hw.getContignuous(path, p_idx)
            if db == p_db:
                link.setChecked(True)

        for elm, params in self.Mutes.items():
            path  = params[0]
            l_elm = params[1]
            r_elm = params[2]

            l_db  = self.hw.getContignuous(path, 1)
            r_db  = self.hw.getContignuous(path, 2)
            l_vol = self.db2vol(l_db)
            r_vol = self.db2vol(r_db)

            if l_vol == r_vol and l_vol == 0:
                elm.setChecked(True)
                l_elm.setDisabled(True)
                r_elm.setDisabled(True)
            elm.clicked.connect(self.updateMute)

        for elm, params in self.Balances.items():
            path = params[0]
            idx  = params[1]
            pan = self.hw.getContignuous(path, idx)
            val = self.pan2val(pan)
            elm.setValue(int(val))
            elm.valueChanged.connect(self.updateBalance)

    # helper functions
    def vol2db(self, vol):
        return (log10(vol + 1) - 2) * 16384
    def db2vol(self, db):
        return pow(10, db / 16384 + 2) - 1

    #       Right - Center - Left
    # 0x8000 - 0x0000 - 0x0001 - 0x7FFE
    #        ..., -1, 0, +1, ...
    def val2pan(self, val):
        return (val - 50) * 0x7ffe / -50
    def pan2val(self, pan):
        return -(pan/ 0x7ffe) * 50 + 50

    def updateSelector(self, state):
        sender = self.sender()
        path   = self.Selectors[sender][0]
        self.hw.setDiscrete(path, state)

    def updateVolume(self, vol):
        sender = self.sender()
        path   = self.Volumes[sender][0]
        idx    = self.Volumes[sender][1]
        pair   = self.Volumes[sender][2]
        link   = self.Volumes[sender][3]
        mute   = self.Volumes[sender][4]
        db = self.vol2db(vol)
        self.hw.setContignuous(path, db, idx)

        if idx == 1:
            p_idx = 2
        else:
            p_idx = 1
        if link.isChecked():
            self.hw.setContignuous(path, db, p_idx)
            pair.setValue(vol)

    def updateMicBoost(self, state):
        sender = self.sender()
        path   = self.MicBoosts[sender][0]
        idx    = self.MicBoosts[sender][1]
        if state:
            value = 0x7ffe
        else:
            value = 0x0000
        self.hw.setContignuous(path, value, idx)

    def updateBalance(self, val):
        sender = self.sender()
        path   = self.Balances[sender][0]
        idx    = self.Balances[sender][1]
        pan    = self.val2pan(val)
        self.hw.setContignuous(path, pan, idx)

    def updateMute(self, state):
        sender = self.sender()
        path   = self.Mutes[sender][0]
        l_elm  = self.Mutes[sender][1]
        r_elm  = self.Mutes[sender][2]

        if state:
            # mute emulation
            db = self.vol2db(10)
            l_elm.setValue(0)
            l_elm.setDisabled(True)
            r_elm.setValue(0)
            r_elm.setDisabled(True)
        else:
            db = self.vol2db(99)
            l_elm.setDisabled(False)
            l_elm.setValue(99)
            r_elm.setDisabled(False)
            r_elm.setValue(99)

        self.hw.setContignuous(path, db, 1)
        self.hw.setContignuous(path, db, 2)
